"""Functions related to file operations"""

from os.path import join, abspath, expanduser
from os import environ, makedirs, walk


def get_sietch_dir() -> str:
    """Get the directory where sietch environments are installed

    Returns
    -------
    str
        The directory where sietch environments are installed
    """

    if "SIETCH_HOME" in environ:
        return environ["SIETCH_HOME"]
    else:
        return abspath(join(expanduser("~"), ".sietch"))


def create_sietch_dir() -> None:
    """Create the sietch directory if it doesn't exist"""

    sietch_dir = get_sietch_dir()
    if not sietch_dir.exists():
        makedirs(sietch_dir)


def get_environment_dir(environment_name: str) -> str:
    """Get the directory for the specified environment

    Parameters
    ----------
    environment_name : str
        The name of the environment

    Returns
    -------
    str
        The directory for the specified environment
    """

    return join(get_sietch_dir(), environment_name)


def create_environment_dir(environment_name: str) -> None:
    """Create the directory for the specified environment

    Parameters
    ----------
    environment_name : str
        The name of the environment
    """

    environment_dir = get_environment_dir(environment_name)
    if not environment_dir.exists():
        makedirs(environment_dir)


def list_environments() -> list:
    """List all environments

    Returns
    -------
    list[str]
        A list of all environments
    """

    envs = [x for x in next(walk(get_sietch_dir()))[1]]

    return envs



"""Main cli entry point for the application"""

import click

from ..environment import (
    create_environment,
    remove_environment,
    prepare_activate_environment,
)
from ..directory import list_environments
from ..cli import parameters as p


@click.group()
def cli():
    """CLI entrypoint"""


@cli.command("create")
@p.name
def create(name: str):
    """Create a new environment

    Parameters
    ----------
    name : str
        The name of the environment to create
    """
    click.echo(f"Creating environment {name}")
    create_environment(name)


@cli.command("remove")
@p.name
def remove(name: str):
    """Remove an environment

    Parameters
    ----------
    name : str
        The name of the environment to remove
    """
    remove_environment(name)


@cli.command("list")
def list_():
    """List all environments"""
    envs = list_environments()

    if len(envs) == 0:
        click.echo("No environments found")
    else:
        click.echo("Environments: \n-------------")
        for env in envs:
            click.echo(f"{env}")


@cli.command("activate")
@p.name
def activate(name: str):
    """Activate an environment.
    NOTE: Does not actually activate the environment, but provides the command to do so.

    Parameters
    ----------
    name : str
        The name of the environment to activate
    """

    click.echo("Run the following command to activate the environment:")
    prepare_activate_environment(name)



"""Setup script for sietch"""

from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="sietch",
    version="0.1.0",
    author="GandalfsDad",
    description="Lightweight environment & cli tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/GandalfsDad/sietch",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
    install_requires=[
        "click",
    ],
    packages=find_packages(),
    entry_points={"console_scripts": ["sietch=sietch.cli.main:cli"]},
)



"""Environment creation functions"""

import subprocess
from os.path import join

from ..directory import get_environment_dir


def create_environment(environment_name: str) -> None:
    """Create a new environment

    Parameters
    ----------
    environment_name : str
        The name of the environment to create
    """
    environment_path = get_environment_dir(environment_name)

    subprocess.run(["python", "-m", "venv", environment_path], check=True)


def remove_environment(environment_name: str) -> None:
    """Remove an environment

    Parameters
    ----------
    environment_name : str
        The name of the environment to remove
    """
    environment_path = get_environment_dir(environment_name)

    subprocess.run(["rm", "-rf", environment_path], check=True)


def prepare_activate_environment(environment_name: str) -> None:
    """Prepare the environment for activation

    Parameters
    ----------
    environment_name : str
        The name of the environment to activate
    """

    environment_path = get_environment_dir(environment_name)
    activate_path = join(environment_path, "bin", "activate")

    return subprocess.run(f"echo 'source {activate_path}'", check=True, shell=True)
